%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IDENTIFICATION OF THE RE-USE OF COLLATERAL BASED ON TRANSACTION DATA

% Algorithm described in: 
% - Lucas Marc Fuhrer, Basil Guggenheim and Silvio Schumacher: Re-use of collateral in the repo market, Journal of Money, Credit and Banking.
% - Lucas Marc Fuhrer, Basil Guggenheim and Silvio Schumacher (2015): Re-use of collateral in the repo market, SNB Working Paper 2015-2.

% Authors: Lucas Marc Fuhrer, Basil Guggenheim and Silvio Schumacher

% MATLAB Version 2014a

% Name of file: "StartFile.m"
% Additional functions:
% - "a_matching.m"
% - "b_preparation.m"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Description:

% 1. Define parameters
% 2. Load and sort dataset 
% 3. Find matches
% 4. Mark transactions in data set and reduce overestimation

%% 1. Define parameters

tic
clc
clear

% A: Define working directory
dirName = 'P:\Collaterals\Matlab\JMCB\Final\';
cd(dirName);

% B: Define path to save output files
path = 'P:\Collaterals\Matlab\JMCB\Final\';

% C: Define name suffix of files and name of input data file
name = 'SampleFile';

%% 2. Load and sort dataset

% Structure of input data, written in matrix with name "Qdata"
% Coloumn: 1: Repo ID; 2: unique ID for ISIN; 3: Purchase Date; 4: Repurchase Date;  5: Cash Taker ID; 6: Cash Provider ID; 7: Cash Amount in CHF; 8: Duration of transaction 9: Rating of ISIN (ID), 10: Repo rate 

loadstr = ['load ' name '.mat'];
eval(loadstr)

% Sort the dataset w.r.t. purchase date, duration and volume
sortColumns = [7,8,3];

for i=1:size(sortColumns,2)
    sid= sortColumns(1,i);
    [~,s_idx]=sort(Qdata(:,sid)); 
    Qdata=Qdata(s_idx,:); 
end 

% Compute unique transaction index
TransID = 1:size(Qdata,1);
TransID = transpose(TransID/size(Qdata,1));


%% 3. Find matches

[Qdata] = a_matching(Qdata,name,path,TransID);


%%  4. Mark transactions in data set and reduce overestimation

[QdataEx,Qdata] = b_preparation(Qdata,name,path);

toc
disp 'Calculation has been successful'

% Structure of output file (called QdataEx)
% Coloumn: 1: Repo ID; 2: unique ID for ISIN; 3: Purchase Date; 4: Repurchase Date;  5: Cash Taker ID; 6: Cash Provider ID; 7: Cash Amount in CHF; 8: Duration of transaction 9: Rating, 10: Repo Rate 
% 11: Value of re-use in transaction i; 12: Sum of total re-use which is based on initial transaction i; 13: Dummy, if there is a re-use of collateral in transaction i; 14: Count of re-used transactions per initial transaction i 

