function[Qdata] = a_matching(Qdata,name,path,TransID)


% 1. Generate order key by purchase date (in ascending order) and duration of the repo transaction (in descending order), i.e. starting with the oldest purchase date and the longest maturity

key = (max(Qdata(:,3)) + 1 - Qdata(:,3))*10000 + (Qdata(:,4) - Qdata(:,3));
[~, ~, order_s] = unique(key + TransID); % If the purchase date and the maturitiy are identical, the transactions are sorted w.r.t. the transaction index (generate in start file)


% 2. Prepare matrices

A1 = sparse(size(Qdata,1),10); % a matrix ("A-Matrix") to write the indices of matched transactions
used = zeros(size(Qdata,1),1); % vector to check whether a transaction has already been used

% 3. Start with iteration over each transaction included in the dataset to check the (main) conditions for re-use

h = waitbar(0,'please wait...');
xxx = size(used,1);                                     % start with the oldest transaction with the longest maturity
while sum(used) < size(used,1) 
    
    line_CT = (order_s == xxx);                         % defines index of the potential initial transaction ("first transaction")   
    used(line_CT) = 1;                                 % this line has now been analysed (ensuring only iteration is done per transaction) 
    line_CT_write = line_CT;
        
    %   4.  Check whether the following conditions hold for all subsequent transactions
    
    line_CP = (Qdata(:,2) == Qdata(line_CT,2)) & ...    ISIN of first transaction is equal to the ISIN of the second transaction
        (Qdata(:,3) >= Qdata(line_CT,3) ) & ...         Purchase date of second transaction later than purchase date of first transaction
        (Qdata(:,4) <= Qdata(line_CT,4) ) & ...         Repurchase date of second transaction not later than repurchase for first transaction
        (Qdata(:,5) == Qdata(line_CT,6)) &...           Cash Provider ID of first transaction is equal to Cash Taker ID of second transaction
        (used == 0) ;                                   % ensuring only iteration is done per transaction
       
    
    %  5. Write the indices in the A-Matrix
    
    used(line_CP) = 1;                                 % this line (transaction) has been used (i.e. there is a re-use and the transaction is not available for an other re-use)
    line_match = unique(line_CP'.*(1:size(used,1)));   % retrieve the indices of the second (re-use) transactions
    line_match = line_match(2:end);                  
    
    A1(line_CT_write,1:numel(line_match)) = line_match; % write the indices of the re-use transactions at the line of the initial transaction
    
    xxx = xxx - 1;                                      % move on to the next line
    waitbar( sum(used) / size(used,1) )
    
end

close(h)


% 6. Save data

savestr =  ['save ',path, 'a_matching_' name '.mat'];
evalc(savestr);


