function[QdataEx,Qdata] = b_preparation(Qdata,name,path)


%% Mark the initial and re-use transactions in the dataset, while reducing the overestimation by having several matches

% Key elements of the script:
% - There might exist several matches per first transaction
%   a) The script checks whether the values of the second transactions do not exceed the value of the first transaction and (STEP 1)
%   b) The script checks whether there are several re-uses at the same time - if so the amount of the second transaction is reduced to the maximum value possible to re-use (STEP 2)

%% Load data from previous step

loadstr = ['load ',path, 'a_matching_' name '.mat'];
evalc(loadstr);


%% Set up a matrix to mark the transactions which collateral were re-used and where it was re-used

QdataEx = Qdata;
QdataEx(:,11) = 0 ;     % Value of re-use in transaction i
QdataEx(:,12) = 0 ;     % Sum of total re-use which is based on initial transaction i
QdataEx(:,13) = 0 ;     % Dummy, if collateral there is a re-use of collateral 
QdataEx(:,14) = 0 ;     % Count all re-used transactions per initial transaction i 

%% STEP 1

% Check whether the value of the second transaction does not exceed the value of the first transaction
% --> Compute the maximum possible value of the second transaction, i.e. the value that is "<=" the value of the first transaction
% --> Extract the purchase date (matA1Start) the repurchase date (matA1End) and the value (matA1Val) of the re-use transaction for Step 2

% Extract necessary information of A-matrix, i.e. indices of first transactions and according second transactions
aa1 = transpose(1:length(A1));
idx = (0<sum(A1(:,1:end),2));
matA1T = full(A1(idx,:));
idxA1 = aa1(idx,:);
matA1T = horzcat(idxA1,matA1T);

matA1Start = matA1T;
matA1End  = matA1T;
matA1Val = matA1T;
matA1ValDiff = matA1T;

% Iterate over all first transactions which severed for a re-use later on 
for i=1:size(matA1T,1);
    valOrg = QdataEx(matA1T(i,1),7);                                   % get value of the first transaction
    for j=2:size(matA1T,2);                                            % iterate over all matched second transactions
        col = matA1T(i,j);                                             
        if (col > 0);                                                     
            matA1Start(i,j) = QdataEx(col,3);                          % retrieve purchase date of first transaction
            matA1End(i,j) = QdataEx(col,4);                            % retrieve repurchase date of first transaction
            if QdataEx(col,7) <= valOrg;                               % check whether the value of the first transaction is at least as big as the second transaction
                matA1Val(i,j) = QdataEx(col,7);                        % if so: retrieve the value of the second transaction
                matA1ValDiff(i,j)  = valOrg - QdataEx(col,7);          % assign the value that is left over for further second transactions running at the same time
            else
                matA1Val(i,j) = valOrg;                                % if else: value of the second transaction is larger than the first transaction, retreive the value of the first transaction
                matA1ValDiff(i,j)  = 0;                                % assign the value of zero that is left over for further second transactions running at the same time
            end
        end
    end
end

%% STEP 2

% Check whether there are several second transaction at the same time and calculate the maximum possible value to be re-used at once

% Set up conflict matrix and book keeping matrizes (for the count/sum of total re-use which is based on initial transaction i)
matA1Con = zeros(size(matA1T,1),size(matA1T,2));
matA1ValFin = zeros(size(matA1T,1),size(matA1T,2));
matA1CountFin = zeros(size(matA1T,1),size(matA1T,2));

% Iterate over all initial transactions
for i=1:size(matA1T,1);
    
    numEle = (matA1Val(i,:) > 0);                                       
    numElements = double(numEle) * transpose(double(numEle));          % number possible second transactions with postive value based on initial transaction i
    idorg = matA1T(i,1);                                               % get index of first transaction 
    
    % iteration #1: i) mark the transaction in the dataset that was re-used and the value of re-use and ii) mark the transaction where the re-use took place and the value of re-use
    QdataEx(matA1T(i,2),11) = matA1Val(i,2);                           % value at second transaction
    QdataEx(idorg,12) = matA1Val(i,2);                                 % value at first transaction
    QdataEx(matA1T(i,2),13) =  1;                                      % dummy at second transaction
    QdataEx(idorg,14) =  1 ;                                           % dummy at first transaction
    
    % iteration #2 - max: check whether second transactions are overlapping (in time) and assign according maximum value (reduce possible overestimation)
    if numElements >= 3;                                               
        
        for j=2:numElements-1                                          % Iteration over subsequent potential re-use transactions                              

            % Compute a "conflict" matrix: check for all potential second transaction whether they are overlapping (i.e. at the same time)
            for k=1:max(numElements-j,1)                               
                if matA1Start(i,j+k) < matA1End(i,j);                  % if start date of next contracts is before end date of reference contract...
                    matA1Con(i,j+k) = 1;                               % ...there is a conflict (=1) --> we need to check whether the value of the inital transaction is sufficiently large for all the re-use transactions
                end
            end
            
            % Assign values 
            for k=1:max(numElements-j,1)                               % starts iterating through possible second transactions
                idx = matA1T(i,j+k);       
               
                if matA1Con(i,j+k) > 0 && matA1ValDiff(i,j) > 0        % Case 1: with a conflict, only the value left (in matA1ValDiff) can be assigned to the second transaction
                    QdataEx(idx,11) = min(matA1Val(i,j+k),matA1ValDiff(i,j)); % Assign value at second transaction, i.e. take the minimum value  of the value left and the value of the second transaction
                    matA1ValFin(i,j+k)  = min(matA1Val(i,j+k),matA1ValDiff(i,j)); % To account later on how much was re-used in value per every first transaction; all linked second transactions are written in a matrix
                    matA1ValDiff(i,j) = max( ( matA1ValDiff(i,j) - min(matA1Val(i,j+k),matA1ValDiff(i,j))),0); % the left over matrix must be adjusted
                    QdataEx(idx,13) = 1;                               % Set dummy variable at second transaction
                    matA1CountFin(i,j+k) = 1;                          % To account later on how many transactions were re-used per every first transaction; the number of second transactions are written in a matrix
                    
                elseif matA1Con(i,j+k) == 0                            % Case 2: without a confilct, the whole value possible can be assigned to the second transaction
                    QdataEx(idx,11) = matA1Val(i,j+k);                 % Assign value at second transaction
                    matA1ValFin(i,j+k) = matA1Val(i,j+k);              % To account later on how much was re-used in value per every first transaction; all linked second transactions are written in a matrix 
                    QdataEx(idx,13) =  1;                              % Set dummy variable at second transaction
                    matA1CountFin(i,j+k) = 1;                          % To account later on how many transactions were re-used per every first transaction; the number of second transactions are written in a matrix
                end
                
                if  matA1ValDiff(i,j) <= 0, break, end                 % if there is no more value to distribute stop the process (no more re-use possible)
                
            end
        end
    end
    
    sumVal = sum(matA1ValFin(i,1:end));                                % Sum over all re-used transactions per first transaction i
    sumCount = sum(matA1CountFin(i,1:end));                            % Count all re-used transactions per first transaction i
    QdataEx(idorg,12) = QdataEx(idorg,12) + sumVal ;                   % Add transaction volumes from first iteration and sum over iteration 2 - max
    QdataEx(idorg,14) = QdataEx(idorg,14)+ sumCount ;                  % Add transactions (count) from first iteration and sum over iteration 2 - max (count) 
    
end


%% Save data

savestr =  ['save ',path, 'b_preparation_' name '.mat'];
evalc(savestr);

